from ..mixins import OptimViewMixin, PaginateStackMixin, OrderingMixin, CamomillaBasePermissionMixin
from rest_framework import viewsets


class BaseModelViewset(
    CamomillaBasePermissionMixin, OptimViewMixin, OrderingMixin, PaginateStackMixin, viewsets.ModelViewSet
):
    pass



from django.db.models.query import QuerySet
from django.core.exceptions import ObjectDoesNotExist
from django.apps import apps

URL_NODE_RELATED_NAME = "%(app_label)s_%(class)s"


class PageQuerySet(QuerySet):

    __UrlNodeModel = None

    @property
    def UrlNodeModel(self):
        if not self.__UrlNodeModel:
            self.__UrlNodeModel = apps.get_model("camomilla", "UrlNode")
        return self.__UrlNodeModel

    def get_permalink_kwargs(self, kwargs):
        return list(set(kwargs.keys()).intersection(set(self.UrlNodeModel.LANG_PERMALINK_FIELDS + ["permalink"])))

    def get(self, *args, **kwargs):
        permalink_args = self.get_permalink_kwargs(kwargs)
        if len(permalink_args):
            try:
                node = self.UrlNodeModel.objects.get(**{arg: kwargs.pop(arg) for arg in permalink_args})
                kwargs["url_node"] = node
            except ObjectDoesNotExist:
                raise self.model.DoesNotExist(
                    "%s matching query does not exist." % self.model._meta.object_name
                )
        return super(PageQuerySet, self).get(*args, **kwargs)



import pytest
import json
import os
from camomilla.models import Media
from .fixtures import load_asset
from .utils.api import login_superuser
from rest_framework.test import APIClient
from django.conf import settings

client = APIClient()


def load_asset_and_remove_media(filename):
    asset = load_asset(filename)
    if os.path.exists(f"{settings.MEDIA_ROOT}/{filename}"):
        os.remove(f"{settings.MEDIA_ROOT}/{filename}")
    return asset


@pytest.mark.django_db
def test_media_api_creation():
    token = login_superuser()
    client.credentials(HTTP_AUTHORIZATION='Token ' + token)
    asset = load_asset_and_remove_media("10595073.png")
    response = client.post(
        "/api/camomilla/media/",
        {
            "file": asset,
            "data": json.dumps({"translations": {"en": {"alt_text": "Test", "title": "Test", "description": "Test"}}}),
        },
        format="multipart",
    )
    assert response.status_code == 201
    assert Media.objects.count() == 1
    media = Media.objects.first()
    assert media.alt_text == "Test"
    assert media.title == "Test"
    assert media.description == "Test"
    assert media.file.name == "10595073.png"


@pytest.mark.django_db
def test_media_compression():
    token = login_superuser()
    client.credentials(HTTP_AUTHORIZATION='Token ' + token)
    asset = load_asset_and_remove_media("Sample-jpg-image-10mb.jpg")
    asset_size = asset.size
    response = client.post(
        "/api/camomilla/media/",
        {
            "file": asset,
            "data": json.dumps({"translations": {"en": {"alt_text": "Test", "title": "Test", "description": "Test"}}}),
        },
        format="multipart",
    )
    assert response.status_code == 201
    assert Media.objects.count() == 1
    media = Media.objects.first()
    assert media.file.size < asset_size
    assert media.file.size < 1000000  # 1MB


@pytest.mark.django_db
def test_inflating_prevent():
    token = login_superuser()
    client.credentials(HTTP_AUTHORIZATION='Token ' + token)
    asset = load_asset_and_remove_media("optimized.jpg")
    asset_size = asset.size
    response = client.post(
        "/api/camomilla/media/",
        {
            "file": asset,
            "data": json.dumps({"translations": {"en": {"alt_text": "Test", "title": "Test", "description": "Test"}}}),
        },
        format="multipart",
    )
    assert response.status_code == 201
    assert Media.objects.count() == 1
    media = Media.objects.first()
    assert media.file.size < asset_size



from enum import Enum
from uuid import uuid4
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _
from django.template.loader import render_to_string
from django.template import RequestContext
from django.utils.safestring import mark_safe
from pydantic import (
    Field,
    SerializationInfo,
    computed_field,
    model_serializer,
)
from structured.pydantic.models import BaseModel
from structured.fields import StructuredJSONField
from camomilla.models.page import UrlNode, AbstractPage
from typing import Optional, Union, Callable, List
from django.db.models.base import Model as DjangoModel
from django.conf import settings


class LinkTypes(str, Enum):
    relational = "RE"
    static = "ST"


class MenuNodeLink(BaseModel):
    link_type: LinkTypes = LinkTypes.static
    static: str = None
    content_type: ContentType = None
    page: AbstractPage = None
    url_node: UrlNode = None

    @model_serializer(mode="wrap", when_used="json")
    def update_relational(self, handler: Callable, info: SerializationInfo):
        if self.link_type == LinkTypes.relational:
            if self.content_type and self.page:
                if isinstance(self.page, DjangoModel) and not self.page._meta.abstract:
                    self.content_type = ContentType.objects.get_for_model(self.page.__class__)
                ctype_id = getattr(self.content_type, "pk", self.content_type)
                page_id = getattr(self.page, "pk", self.page)
                c_type = ContentType.objects.filter(pk=ctype_id).first()
                model = c_type and c_type.model_class()
                page = model and model.objects.filter(pk=page_id).first()
                self.url_node = page and page.url_node
            elif self.url_node:
                url_node_id = getattr(self.url_node, "pk", self.url_node)
                self.page = UrlNode.objects.filter(pk=url_node_id).first().page
                self.content_type = ContentType.objects.get_for_model(self.page.__class__)
        return handler(self)

    def get_url(self, request=None):
        if self.link_type == LinkTypes.relational:
            return isinstance(self.url_node, UrlNode) and self.url_node.routerlink
        elif self.link_type == LinkTypes.static:
            return self.static

    @computed_field
    @property
    def url(self) -> Optional[str]:
        return self.get_url()


class MenuNode(BaseModel):
    id: str = Field(default_factory=uuid4)
    meta: dict = {}
    nodes: List["MenuNode"] = []
    title: str = ""
    link: MenuNodeLink


class Menu(models.Model):
    key = models.CharField(max_length=200, unique=True, editable=False)
    available_classes = models.JSONField(default=dict, editable=False)
    enabled = models.BooleanField(default=True)
    nodes = StructuredJSONField(default=list, schema=MenuNode)

    class Meta:
        verbose_name = _("menu")
        verbose_name_plural = _("menus")

    def render(
        self,
        template_path: str,
        request=None,
        context: Union[dict, RequestContext] = {},
    ):
        if isinstance(context, RequestContext):
            context = context.flatten()
        is_preview = bool(request.GET.get("preview", False))
        context.update({"menu": self, "is_preview": is_preview})
        return mark_safe(render_to_string(template_path, context, request))

    class defaultdict(dict):
        def __missing__(self, key):
            dict.__setitem__(self, key, Menu.objects.get_or_create(key=key)[0])
            return self[key]

    def __str__(self) -> str:
        return self.key
